@@ -32,6 +32,7 @@ from .change_tracker import setFigureVariableNames
 from .drag_helper import DragManager
 from .exception_swallower import swallow_get_exceptions
 from .matplotlibwidget import MatplotlibWidget
+from .helper_functions import convertFromPyplot
 
 
 def my_excepthook(type, value, tback):
@@ -74,8 +75,8 @@ def initialize(use_global_variable_names=False):
     plt.figure = figure
     patchColormapsWithMetaInfo()
 
-    stack_call_position = traceback.extract_stack()[-2]
-    stack_call_position.filename
+    #stack_call_position = traceback.extract_stack()[-2]
+    #stack_call_position.filename
 
     plt.keys_for_lines = keys_for_lines
 
@@ -88,6 +89,13 @@ def initialize(use_global_variable_names=False):
 
     Figure.savefig = savefig
 
+    # iterate over figures
+    for fig_number in _pylab_helpers.Gcf.figs.copy():
+        fig_old = plt.figure(fig_number)
+        fig = plt.figure(fig_number, force_add=True)
+        convertFromPyplot(fig_old, fig)
+        plt.close(fig_old)
+
 
 def show(hide_window: bool = False):
     """ the function overloads the matplotlib show function.
@@ -100,7 +108,7 @@ def show(hide_window: bool = False):
         myappid = 'rgerum.pylustrator'  # arbitrary string
         ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
     # iterate over figures
-    for figure in _pylab_helpers.Gcf.figs:
+    for figure in _pylab_helpers.Gcf.figs.copy():
         # get variable names that point to this figure
         if setting_use_global_variable_names:
             setFigureVariableNames(figure)
@@ -146,14 +154,14 @@ def patchColormapsWithMetaInfo():
     Colormap.__call__ = new_call
 
 
-def figure(num=None, figsize=None, *args, **kwargs):
+def figure(num=None, figsize=None, force_add=False, *args, **kwargs):
     """ overloads the matplotlib figure call and wrapps the Figure in a PlotWindow """
     global figures
     # if num is not defined create a new number
     if num is None:
         num = len(_pylab_helpers.Gcf.figs) + 1
     # if number is not defined
-    if num not in _pylab_helpers.Gcf.figs.keys():
+    if force_add or num not in _pylab_helpers.Gcf.figs.keys():
         # create a new window and store it
         canvas = PlotWindow(num, figsize, *args, **kwargs).canvas
         canvas.figure.number = num
@@ -605,9 +613,9 @@ class Align(QtWidgets.QWidget):
         self.layout = QtWidgets.QHBoxLayout(self)
         self.layout.setContentsMargins(0, 0, 0, 0)
 
-        actions = ["left_x", "center_x", "right_x", "distribute_x", "top_y", "center_y", "bottom_y", "distribute_y"]
+        actions = ["left_x", "center_x", "right_x", "distribute_x", "top_y", "center_y", "bottom_y", "distribute_y", "group"]
         icons = ["left_x.png", "center_x.png", "right_x.png", "distribute_x.png", "top_y.png", "center_y.png",
-                 "bottom_y.png", "distribute_y.png"]
+                 "bottom_y.png", "distribute_y.png", "group.png"]
         self.buttons = []
         for index, act in enumerate(actions):
             button = QtWidgets.QPushButton(QtGui.QIcon(os.path.join(os.path.dirname(__file__), "icons", icons[index])),
@@ -1115,3 +1123,4 @@ class PlotWindow(QtWidgets.QWidget):
                 event.ignore()
             if reply == QtWidgets.QMessageBox.Yes:
                 self.fig.change_tracker.save()
+                # app.clipboard().setText("\r\n".join(output))
