@@ -324,6 +324,24 @@ def loadFigureFromFile(filename: str, figure: Figure = None, offset: list = None
             addContentToFigure(figure, axes2)
 
 
+# helper_functions.py
+def convertFromPyplot(old, new):
+
+    w, h = old.get_size_inches()
+    new.set_size_inches(w, h)
+
+    str(new)  # important! (for some reason I don't know)
+    for ax in old.axes:
+        old.delaxes(ax)
+        new._axstack.add(new._make_key(ax), ax)
+        new.bbox._parents.update(old.bbox._parents)
+        new.dpi_scale_trans._parents.update(old.dpi_scale_trans._parents)
+        replace_all_refs(old.bbox, new.bbox)
+        replace_all_refs(old.dpi_scale_trans, new.dpi_scale_trans)
+        replace_all_refs(old.canvas, new.canvas)
+        replace_all_refs(old, new)
+
+
 def mark_inset(parent_axes: Axes, inset_axes: Axes, loc1: Union[int, Sequence[int]] = 1, loc2: Union[int, Sequence[int]] = 2, **kwargs):
     """ like the mark_inset function from matplotlib, but loc can also be a tuple """
     from mpl_toolkits.axes_grid1.inset_locator import TransformedBbox, BboxPatch, BboxConnector
@@ -517,3 +535,94 @@ def add_letters(*args, **kwargs):
     """ add a letter indicating which subplot it is to all of the axes of the given figure """
     for ax in plt.gcf().axes:
         add_letter(ax, *args, **kwargs)
+
+
+def axes_to_grid(axes=None, track_changes=False):
+    # get the axes list
+    if axes is None:
+        fig = plt.gcf()
+        axes = fig.axes
+
+    # get width and heights
+    width = np.mean([ax.get_position().width for ax in axes])
+    height = np.mean([ax.get_position().height for ax in axes])
+    dims = [width, height]
+
+    # group the axes to positions on a grid
+    pos = [[], []]
+    axes_indices = []
+    for ax in axes:
+        center = np.mean(ax.get_position().get_points(), axis=0)
+        new_indices = [0, 0]
+        for i in [0, 1]:
+            d = np.abs(pos[i] - center[i])
+            if len(d) == 0 or np.min(d) > dims[i]/2:
+                pos[i].append(center[i])
+                new_indices[i] = len(pos[i])-1
+            else:
+                new_indices[i] = np.argmin(d)
+        axes_indices.append(new_indices)
+    # sort the indices
+    pos = np.array(pos)
+    for i in [0, 1]:
+        sorted_indices = np.argsort(pos[i], axis=0)
+        for indices in axes_indices:
+            indices[i] = sorted_indices[indices[i]]
+
+    # the count of plots
+    x_count = np.max([i[0] for i in axes_indices])
+    y_count = np.max([i[1] for i in axes_indices])
+
+    # extent of the whole plot area
+    x_min = np.min([ax.get_position().get_points()[0][0] for ax in axes])
+    x_max = np.max([ax.get_position().get_points()[1][0] for ax in axes])
+    y_min = np.min([ax.get_position().get_points()[0][1] for ax in axes])
+    y_max = np.max([ax.get_position().get_points()[1][1] for ax in axes])
+
+    # the space between plots
+    if x_count == 0:
+        x_gap = 0
+    else:
+        x_gap = ((x_max-x_min)-(x_count+1)*width)/x_count
+    if y_count == 0:
+        y_gap = 0
+    else:
+        y_gap = ((y_max-y_min)-(y_count+1)*height)/y_count
+
+    # make all the plots the same size and align them on the grid
+    for i, ax in enumerate(axes):
+        ax.set_position([x_min+axes_indices[i][0] * (width+x_gap),
+                        y_min+axes_indices[i][1] * (height + y_gap),
+                        width,
+                        height,
+                        ])
+        if track_changes is True:
+            ax.figure.change_tracker.addChange(ax, ".set_position([%f, %f, %f, %f])" % (x_min+axes_indices[i][0] * (width+x_gap), y_min+axes_indices[i][1] * (height + y_gap), width, height))
+
+    # make all the plots have the same limits
+    xmin = np.min([ax.get_xlim()[0] for ax in axes])
+    xmax = np.max([ax.get_xlim()[1] for ax in axes])
+    ymin = np.min([ax.get_ylim()[0] for ax in axes])
+    ymax = np.max([ax.get_ylim()[1] for ax in axes])
+    for ax in axes:
+        ax.set_xlim(xmin, xmax)
+        ax.set_ylim(ymin, ymax)
+
+    # hide ticks and labels on plots that are not on the left or bottom
+    for i, ax in enumerate(axes):
+        if axes_indices[i][0] != 0:
+            ax.set_ylabel("")
+            ax.set_yticklabels([])
+            if track_changes is True:
+                ax.figure.change_tracker.addChange(ax, ".get_yaxis().get_label().set_text('')")
+                ax.figure.change_tracker.addChange(ax, ".set_yticklabels([])")
+        if axes_indices[i][1] != 0:
+            ax.set_xlabel("")
+            ax.set_xticklabels([])
+            if track_changes is True:
+                ax.figure.change_tracker.addChange(ax, ".get_xaxis().get_label().set_text('')")
+                ax.figure.change_tracker.addChange(ax, ".set_xticklabels([])")
+        despine(ax)
+        if track_changes is True:
+            ax.figure.change_tracker.addChange(ax, ".spines['right'].set_visible(False)")
+            ax.figure.change_tracker.addChange(ax, ".spines['top'].set_visible(False)")
